@@ -38,7 +38,6 @@ class TestModels(unittest.TestCase):
         self.assertIsInstance(db_creature, models.Creature)
         self.assertEqual(db_creature.name, creature.name)
         self.assertEqual(db_creature.modifier_earth, creature.modifier_earth)
-        self.assertGreater(len(db_creature.loot), 0)
 
         # Dynamic properties
         self.assertEqual(50, db_creature.charm_points)
@@ -46,6 +45,10 @@ class TestModels(unittest.TestCase):
         self.assertEqual(3, len(db_creature.immune_to))
         self.assertEqual(4, len(db_creature.resistant_to))
         self.assertEqual(2, len(db_creature.weak_to))
+        self.assertEqual(9, len(db_creature.abilities))
+        self.assertEqual(2500, db_creature.bestiary_kills)
+        self.assertEqual(50, db_creature.charm_points)
+        self.assertIsNotNone(db_creature.max_damage)
 
         db_creature = models.Creature.get_by_field(self.conn, "name", "demon", use_like=True)
         self.assertIsInstance(db_creature, models.Creature)
@@ -132,11 +135,11 @@ class TestModels(unittest.TestCase):
         self.assertIsInstance(db_item, models.Item)
 
     def test_item_sounds(self):
-        article = Article(1, "Goromaphone", timestamp="2018-08-20T04:33:15Z",
+        article = Article(1, "Mini NabBot", timestamp="2018-08-20T04:33:15Z",
                           content=load_resource("content_item_sounds.txt"))
         item = models.Item.from_article(article)
         self.assertIsInstance(item, models.Item)
-        self.assertEqual(len(item.sounds), 6)
+        self.assertEqual(len(item.sounds), 5)
 
         item.insert(self.conn)
         db_item = models.Item.get_by_field(self.conn, "article_id", 1)
@@ -144,10 +147,24 @@ class TestModels(unittest.TestCase):
         self.assertIsInstance(db_item, models.Item)
         self.assertEqual(db_item.name, item.name)
 
-        db_item = models.Item.get_by_field(self.conn, "name", "goromaphone", use_like=True)
+        db_item = models.Item.get_by_field(self.conn, "name", "mini nabbot", use_like=True)
         self.assertEqual(len(item.sounds), len(db_item.sounds))
         self.assertIsInstance(db_item, models.Item)
 
+    def test_item_store(self):
+        article = Article(1, "Health Potion", timestamp="2018-08-20T04:33:15Z",
+                          content=load_resource("content_item_store.txt"))
+        item = models.Item.from_article(article)
+        self.assertIsInstance(item, models.Item)
+        self.assertEqual(2, len(item.store_offers))
+
+    def test_item_perfect_shot(self):
+        article = Article(1, "Gilded Eldritch Wand", timestamp="2018-08-20T04:33:15Z",
+                          content=load_resource("content_item_perfect_shot.txt"))
+        item = models.Item.from_article(article)
+        self.assertIsInstance(item, models.Item)
+        self.assertEqual(11, len(item.attributes))
+
     def test_key(self):
         article = Article(1, "Key 3940", timestamp="2018-08-20T04:33:15Z",
                           content=load_resource("content_key.txt"))
@@ -163,10 +180,27 @@ class TestModels(unittest.TestCase):
         db_key = models.Key.get_by_field(self.conn, "number", 3940)
         self.assertIsInstance(db_key, models.Key)
 
+    def test_book(self):
+        article = Article(1, "Imperial Scripts (Book)", timestamp="2018-08-20T04:33:15Z",
+                          content=load_resource("content_book.txt"))
+        book = models.Book.from_article(article)
+        self.assertIsInstance(book, models.Book)
+
+        book.insert(self.conn)
+        db_book = models.Book.get_by_field(self.conn, "article_id", 1)
+
+        self.assertIsInstance(db_book, models.Book)
+        self.assertEqual(db_book.name, book.name)
+
     def test_npc(self):
         article = Article(1, "Yaman", timestamp="2018-08-20T04:33:15Z",
                           content=load_resource("content_npc.txt"))
         npc = models.Npc.from_article(article)
+
+        self.assertEqual(1, len(npc.jobs))
+        self.assertEqual("Shopkeeper", npc.job)
+        self.assertEqual(1, len(npc.races))
+        self.assertEqual("Djinn", npc.race)
         self.assertIsInstance(npc, models.Npc)
 
         npc.insert(self.conn)
@@ -243,10 +277,10 @@ class TestModels(unittest.TestCase):
                           content=load_resource("content_charm.txt"))
         charm = models.Charm.from_article(article)
         self.assertIsInstance(charm, models.Charm)
-        self.assertEqual(charm.cost, 900)
-        self.assertEqual(charm.type, "Offensive")
+        self.assertEqual(900, charm.cost)
+        self.assertEqual("Offensive", charm.type)
         self.assertIsInstance(charm.effect, str)
-        self.assertEqual(charm.version, "11.50")
+        self.assertEqual("11.50.6055", charm.version)
 
         charm.insert(self.conn)
         db_charm = models.Charm.get_by_field(self.conn, "article_id", 1)
