@@ -12,13 +12,15 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+"""Contains all the models representing TibiaWiki articles."""
+
 from tibiawikisql.models.abc import Parseable, Row
 from tibiawikisql.models.achievement import Achievement
 from tibiawikisql.models.charm import Charm
-from tibiawikisql.models.creature import Creature, CreatureDrop
+from tibiawikisql.models.creature import Creature, CreatureAbility, CreatureDrop, CreatureMaxDamage, CreatureSound
 from tibiawikisql.models.house import House
 from tibiawikisql.models.imbuement import Imbuement, ImbuementMaterial
-from tibiawikisql.models.item import Item, ItemAttribute, Key
+from tibiawikisql.models.item import Book, Item, ItemAttribute, ItemStoreOffer, Key
 from tibiawikisql.models.mount import Mount
 from tibiawikisql.models.npc import Npc, NpcBuyOffer, NpcDestination, NpcOffer, NpcSellOffer, NpcSpell, RashidPosition
 from tibiawikisql.models.outfit import Outfit, OutfitImage, OutfitQuest
