@@ -13,15 +13,19 @@
 #  limitations under the License.
 
 import re
+import sqlite3
 from collections import OrderedDict
-from typing import List, Optional
+from typing import List, Optional, TYPE_CHECKING
+
+import mwparserfromhell
 
 from tibiawikisql import schema
 from tibiawikisql.models import abc
 from tibiawikisql.utils import (clean_links, int_pattern, parse_boolean, parse_integer, parse_min_max, parse_sounds,
-                                clean_question_mark)
+                                clean_question_mark, strip_code, find_template)
 
-creature_loot_pattern = re.compile(r"\|{{Loot Item\|(?:([\d?+-]+)\|)?([^}|]+)")
+if TYPE_CHECKING:
+    from mwparserfromhell.nodes import Template
 
 KILLS = {
     "Harmless": 25,
@@ -44,6 +48,102 @@ CHARM_POINTS = {
 ELEMENTAL_MODIFIERS = ["physical", "earth", "fire", "ice", "energy", "death", "holy", "drown", "hpdrain"]
 
 
+def parse_abilities(value):
+    """Parse the abilities of a creature.
+
+    Parameters
+    ----------
+    value: :class:`str`
+        A string containing the creature's abilities definition.
+
+    Returns
+    -------
+    :class:`list` of :class:`dict`
+        A list of dictionaries with the ability data.
+    """
+    if not value:
+        return []
+    parsed = mwparserfromhell.parse(value)
+    ability_list_template = find_template(value, "Ability List")
+    if not ability_list_template:
+        return [{
+            "name": strip_code(parsed),
+            "element": "no_template",
+        }]
+    abilities = []
+    for element in ability_list_template.params:
+        ability_template = next(element.value.ifilter_templates(recursive=False), None)
+        if not ability_template:
+            abilities.append({
+                "name": strip_code(element),
+                "element": "plain_text",
+            })
+            continue
+        template_name = str(ability_template.name)
+        ability = None
+        if template_name == "Melee":
+            ability = {
+                "name": ability_template.get("name", "Melee"),
+                "effect": ability_template.get(1, ability_template.get("damage", "?")),
+                "element": ability_template.get(2, ability_template.get("element", "physical")),
+            }
+        if template_name == "Summon":
+            ability = {
+                "name": ability_template.get(1, ability_template.get("creature", None)),
+                "effect": ability_template.get(2, ability_template.get("amount", '1')),
+                "element": "summon",
+            }
+        if template_name == "Healing":
+            ability = {
+                "name": ability_template.get(1, ability_template.get("name", "Self-Healing")),
+                "effect": ability_template.get(2, ability_template.get("range", ability_template.get("damage", "?"))),
+                "element": "healing",
+            }
+        if template_name == "Ability":
+            ability = {
+                "name": ability_template.get(1, ability_template.get("name", None)),
+                "effect": ability_template.get(2, ability_template.get("damage", "?")),
+                "element": ability_template.get(3, ability_template.get("element", "physical")),
+            }
+            if ability["name"] is None:
+                ability["name"] = f'{ability["element"].title()} Damage'
+        if ability:
+            abilities.append(strip_code(ability))
+    return abilities
+
+
+def parse_maximum_damage(value):
+    """Parse the maximum damage template from TibiaWiki.
+
+    If no template is found, the highest number found is considered the total damage.
+
+    Parameters
+    ----------
+    value: :class:`str`
+        A string containing the creature's max damage.
+
+    Returns
+    -------
+    :class:`dict`
+        A dictionary containing the maximum damage by element if available.
+    """
+    if not value:
+        return None
+    max_damage_template = find_template(value, "Max Damage")
+    if not max_damage_template:
+        total = parse_maximum_integer(value)
+        if total is None:
+            return None
+        return {"total": parse_maximum_integer(value)}
+    damages = {}
+    for element in max_damage_template.params:
+        damages[strip_code(element.name).lower()] = parse_integer(strip_code(element.value), -1)
+    excluded = {"summons", "manadrain"}
+    if "total" not in damages:
+        damages["total"] = sum(v for k, v in damages.items() if k not in excluded)
+    return damages
+
+
 def parse_maximum_integer(value):
     """
     From a string, finds the highest integer found.
@@ -58,8 +158,6 @@ def parse_maximum_integer(value):
     :class:`int`, optional:
         The highest number found, or None if no number is found.
     """
-    if value is None:
-        return None
     matches = int_pattern.findall(value)
     try:
         return max(list(map(int, matches)))
@@ -68,8 +166,7 @@ def parse_maximum_integer(value):
 
 
 def parse_loot(value):
-    """
-    Gets every item drop entry of a creature's drops.
+    """Get every item drop entry of a creature's drops.
 
     Parameters
     ----------
@@ -81,12 +178,23 @@ def parse_loot(value):
     tuple:
         A tuple containing the amounts and the item name.
     """
-    return creature_loot_pattern.findall(value)
+    def match(k):
+        return "Item" in k.name
+    loot_items_templates: List['Template'] = mwparserfromhell.parse(value).filter_templates(recursive=True,
+                                                                                            matches=match)
+    loot = []
+    for item_template in loot_items_templates:
+        param_count = len(item_template.params)
+        if param_count < 3:
+            loot.append((strip_code(item_template.get(1)), ""))
+        else:
+            loot.append((strip_code(item_template.get(2)), (strip_code(item_template.get(1)))))
+    return loot
 
 
 def parse_monster_walks(value):
-    """
-    Matches the values against a regex to filter typos or bad data on the wiki.
+    """Match the values against a regex to filter typos or bad data on the wiki.
+
     Element names followed by any character that is not a comma will be considered unknown and will not be returned.
 
     Examples:
@@ -136,7 +244,7 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
         The race name of the creature in Tibia.com's library.
     creature_class: :class:`str`
         The creature's classification.
-    type: :class:`str`
+    creature_type: :class:`str`
         The creature's type.
     type_secondary: :class:`str`
         The creature's secondary type, if any.
@@ -154,8 +262,6 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
         The creature's armor value.
     speed: :class:`int`
         The creature's speed value.
-    max_damage: :class:`int`
-        The maximum amount of damage the creature can do in a single turn.
     runs_at: :class:`int`
         The amount of hitpoints when the creature starts to run away. 0 means it won't run away.
     summon_cost: :class:`int`
@@ -194,8 +300,10 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
         The percentage of damage received of life drain damage. ``None`` if unknown.
     modifier_healing: :class:`int`
         The healing modifier. ``None`` if unknown.
-    abilities: :class:`str`
-        A brief description of the creature's abilities.
+    abilities: list of :class:`CreatureAbility`
+        A list of the creature abilities.
+    max_damage: :class:`CreatureMaxDamage`
+        The maximum damage the creature can make.
     walks_through: :class:`str`
         The field types the creature will walk through, separated by commas.
     walks_around: :class:`str`
@@ -211,6 +319,7 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
     loot: list of :class:`CreatureDrop`
         The items dropped by this creature.
     """
+
     _map = {
         "article": ("article", str.strip),
         "name": ("name", str.strip),
@@ -221,13 +330,12 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
         "bestiaryclass": ("bestiary_class", str.strip),
         "bestiarylevel": ("bestiary_level", str.strip),
         "occurrence": ("bestiary_occurrence", str.strip),
-        "primarytype": ("type", str.strip),
+        "primarytype": ("creature_type", str.strip),
         "secondarytype": ("type_secondary", str.strip),
         "hp": ("hitpoints", lambda x: parse_integer(x, None)),
         "exp": ("experience", lambda x: parse_integer(x, None)),
         "armor": ("armor", lambda x: parse_integer(x, None)),
         "speed": ("speed", lambda x: parse_integer(x, None)),
-        "maxdmg": ("max_damage", parse_maximum_integer),
         "runsat": ("runs_at", parse_integer),
         "summon": ("summon_cost", parse_integer),
         "convince": ("convince_cost", parse_integer),
@@ -247,15 +355,13 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
         "drownDmgMod": ("modifier_drown", parse_integer),
         "hpDrainDmgMod": ("modifier_hpdrain", parse_integer),
         "healMod": ("modifier_healing", parse_integer),
-        "abilities": ("abilities", clean_links),
         "walksthrough": ("walks_through", parse_monster_walks),
         "walksaround": ("walks_around", parse_monster_walks),
         "location": ("location", clean_links),
         "implemented": ("version", str.strip),
         "status": ("status", str.lower),
     }
-
-    _pattern = re.compile(r"Infobox[\s_]Creature")
+    _template = "Infobox_Creature"
 
     __slots__ = (
         "article_id",
@@ -266,7 +372,7 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
         "plural",
         "library_race",
         "creature_class",
-        "type",
+        "creature_type",
         "type_secondary",
         "bestiary_level",
         "bestiary_class",
@@ -355,8 +461,7 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
 
     @classmethod
     def from_article(cls, article):
-        """
-        Parses an article into a TibiaWiki model.
+        """Parse an article into a TibiaWiki model.
 
         This method is overridden to parse extra attributes like loot.
 
@@ -376,7 +481,7 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
         if "loot" in creature._raw_attributes:
             loot = parse_loot(creature._raw_attributes["loot"])
             loot_items = []
-            for amounts, item in loot:
+            for item, amounts in loot:
                 if not amounts:
                     _min, _max = 0, 1
                 else:
@@ -387,11 +492,19 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
             sounds = parse_sounds(creature._raw_attributes["sounds"])
             if sounds:
                 creature.sounds = [CreatureSound(creature_id=creature.article_id, content=sound) for sound in sounds]
+        if "abilities" in creature._raw_attributes:
+            abilities = parse_abilities(creature._raw_attributes["abilities"])
+            if abilities:
+                creature.abilities = [CreatureAbility(creature_id=creature.article_id, **ability)
+                                      for ability in abilities]
+        if "maxdmg" in creature._raw_attributes:
+            max_damage = parse_maximum_damage(creature._raw_attributes["maxdmg"])
+            if max_damage:
+                creature.max_damage = CreatureMaxDamage(creature_id=creature.article_id, **max_damage)
         return creature
 
     def insert(self, c):
-        """
-        Inserts the current model into its respective database.
+        """Insert the current model into its respective database.
 
         This method is overridden to insert elements of child rows.
 
@@ -405,6 +518,11 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
             attribute.insert(c)
         for attribute in getattr(self, "sounds", []):
             attribute.insert(c)
+        for attribute in getattr(self, "abilities", []):
+            attribute.insert(c)
+        max_damage = getattr(self, "max_damage", None)
+        if max_damage:
+            max_damage.insert(c)
 
     @classmethod
     def get_by_field(cls, c, field, value, use_like=False):
@@ -413,12 +531,60 @@ class Creature(abc.Row, abc.Parseable, table=schema.Creature):
             return None
         creature.loot = CreatureDrop.search(c, "creature_id", creature.article_id, sort_by="chance", ascending=False)
         creature.sounds = CreatureSound.search(c, "creature_id", creature.article_id)
+        creature.abilities = CreatureAbility.search(c, "creature_id", creature.article_id)
+        creature.max_damage = CreatureMaxDamage.get_by_field(c, "creature_id", creature.article_id)
         return creature
 
 
-class CreatureDrop(abc.Row, table=schema.CreatureDrop):
+class CreatureAbility(abc.Row, table=schema.CreatureAbility):
+    """Represents a creature's ability.
+
+    Attributes
+    ----------
+    creature_id: :class:`int`
+        The article ID of the creature this ability belongs to.
+    name: :class:`str`
+        The name of the ability.
+    effect: :class:`str`
+        The effect of the ability, or the damage range.
+    element: :class:`str`
+        The element of damage type of the ability. This could also be a status condition instead.
+        For abilities that are just plain text, ``plain_text`` is set.
+        For abilities that are not using the abilities templates in TibiaWiki, they are saved as a single entry
+        with element: ``no_template``.
     """
-    Represents an item dropped by a creature.
+
+    __slots__ = (
+        'creature_id',
+        'name',
+        'effect',
+        'element',
+    )
+
+    def __repr__(self):
+        attributes = []
+        for attr in self.__slots__:
+            try:
+                v = getattr(self, attr)
+                if v is None:
+                    continue
+                attributes.append(f"{attr}={v!r}")
+            except AttributeError:
+                pass
+        return f"{self.__class__.__name__}({','.join(attributes)})"
+
+    def insert(self, c):
+        columns = {
+            'creature_id': self.creature_id,
+            'name': self.name,
+            'effect': self.effect,
+            'element': self.element,
+        }
+        self.table.insert(c, **columns)
+
+
+class CreatureDrop(abc.Row, table=schema.CreatureDrop):
+    """Represents an item dropped by a creature.
 
     Attributes
     ----------
@@ -437,6 +603,7 @@ class CreatureDrop(abc.Row, table=schema.CreatureDrop):
     chance: :class:`float`
         The chance percentage of getting this item dropped by this creature.
     """
+
     __slots__ = (
         "creature_id",
         "creature_title",
@@ -465,7 +632,7 @@ class CreatureDrop(abc.Row, table=schema.CreatureDrop):
         return f"{self.__class__.__name__}({','.join(attributes)})"
 
     def insert(self, c):
-        """Inserts the current model into its respective database.
+        """Insert the current model into its respective database.
 
         Overridden to insert using a subquery to get the item's id from the name.
 
@@ -477,9 +644,12 @@ class CreatureDrop(abc.Row, table=schema.CreatureDrop):
         if getattr(self, "item_id", None):
             super().insert(c)
         else:
-            query = f"""INSERT INTO {self.table.__tablename__}(creature_id, item_id, min, max)
-                        VALUES(?, (SELECT article_id from item WHERE title = ?), ?, ?)"""
-            c.execute(query, (self.creature_id, self.item_title, self.min, self.max))
+            try:
+                query = f"""INSERT INTO {self.table.__tablename__}(creature_id, item_id, min, max)
+                            VALUES(?, (SELECT article_id from item WHERE title = ?), ?, ?)"""
+                c.execute(query, (self.creature_id, self.item_title, self.min, self.max))
+            except sqlite3.IntegrityError:
+                return
 
     @classmethod
     def _is_column(cls, name):
@@ -493,9 +663,102 @@ class CreatureDrop(abc.Row, table=schema.CreatureDrop):
                    LEFT JOIN item ON item.article_id = item_id"""
 
 
-class CreatureSound(abc.Row, table=schema.CreatureSound):
+class CreatureMaxDamage(abc.Row, table=schema.CreatureMaxDamage):
+    """Represent a creature's max damage, broke down by damage type.
+
+    Attributes
+    ----------
+    creature_id: :class:`int`
+        The article ID of the creature this max damage belongs to.
+    physical: :class:`int`, optional
+        The maximum physical damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    fire: :class:`int`, optional
+        The maximum fire damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    ice: :class:`int`, optional
+        The maximum ice damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    earth: :class:`int`, optional
+        The maximum earth damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    energy: :class:`int`, optional
+        The maximum energy damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    holy: :class:`int`, optional
+        The maximum holy damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    death: :class:`int`, optional
+        The maximum death damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    drown: :class:`int`, optional
+        The maximum drown damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    lifedrain: :class:`int`, optional
+        The maximum life drain damage dealt by the creature.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    manadrain: :class:`int`, optional
+        The maximum mana drain damage dealt by the creature. This is not counted as part of the total.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    summons: :class:`int`, optional
+        The maximum damage dealt by the creature's summons.
+        If it is unknown, but the creature does deal damage, it will be -1.
+    total: :class:`int`, optional
+        The maximum damage the creature can deal in a single turn.
+        This doesn't count manadrain and summon damage.
+        In most cases, this is simply the sum of the other damages, but in some cases, the amount may be different.
+        If it is unknown, but the creature does deal damage, it will be -1.
     """
-    Represents a sound made by a creature.
+
+    __slots__ = (
+        'creature_id',
+        'physical',
+        'earth',
+        'fire',
+        'ice',
+        'energy',
+        'death',
+        'holy',
+        'drown',
+        'lifedrain',
+        'manadrain',
+        'summons',
+        'total',
+    )
+
+    def __repr__(self):
+        attributes = []
+        for attr in self.__slots__:
+            try:
+                v = getattr(self, attr)
+                if not v:
+                    continue
+                attributes.append(f"{attr}={v!r}")
+            except AttributeError:
+                pass
+        return f"{self.__class__.__name__}({','.join(attributes)})"
+
+    def insert(self, c):
+        columns = {
+            'creature_id': self.creature_id,
+            'physical': self.physical,
+            'earth': self.earth,
+            'fire': self.fire,
+            'ice': self.ice,
+            'energy': self.energy,
+            'death': self.death,
+            'holy': self.holy,
+            'drown': self.drown,
+            'lifedrain': self.lifedrain,
+            'manadrain': self.manadrain,
+            'summons': self.summons,
+            'total': self.total,
+        }
+        self.table.insert(c, **columns)
+
+
+class CreatureSound(abc.Row, table=schema.CreatureSound):
+    """Represents a sound made by a creature.
 
     Attributes
     ----------
@@ -504,6 +767,7 @@ class CreatureSound(abc.Row, table=schema.CreatureSound):
     content: :class:`str`
         The content of the sound.
     """
+
     __slots__ = ("creature_id", "content")
 
     def __init__(self, **kwargs):
