@@ -19,8 +19,8 @@ from tibiawikisql.models import abc
 from tibiawikisql.models.creature import CreatureDrop
 from tibiawikisql.models.npc import NpcBuyOffer, NpcSellOffer
 from tibiawikisql.models.quest import QuestReward
-from tibiawikisql.utils import (clean_links, clean_question_mark, client_color_to_rgb, parse_boolean, parse_float,
-                                parse_integer, parse_sounds)
+from tibiawikisql.utils import (clean_links, clean_question_mark, client_color_to_rgb, find_templates, parse_boolean,
+                                parse_float, parse_integer, parse_sounds, strip_code)
 
 ELEMENTAL_RESISTANCES = ['physical%', 'earth%', 'fire%', 'energy%', 'ice%', 'holy%', 'death%', 'drowning%']
 
@@ -66,7 +66,7 @@ class Item(abc.Row, abc.Parseable, table=schema.Item):
         The item's weight in ounces.
     item_class: :class:`str`
         The item class the item belongs to.
-    type: :class:`str`
+    item_type: :class:`str`
         The item's type.
     type_secondary: :class:`str`
         The item's secondary type, if any.
@@ -109,8 +109,8 @@ class Item(abc.Row, abc.Parseable, table=schema.Item):
         "npcvalue": ("value_sell", parse_integer),
         "npcprice": ("value_buy", parse_integer),
         "flavortext": ("flavor_text", str.strip),
-        "itemclass": ("item_class", str.strip),
-        "primarytype": ("type", str.strip),
+        "objectclass": ("item_class", str.strip),
+        "primarytype": ("item_type", str.strip),
         "secondarytype": ("type_secondary", str.strip),
         "lightcolor": ("light_color", lambda x: client_color_to_rgb(parse_integer(x))),
         "lightradius": ("light_radius", parse_integer),
@@ -118,7 +118,7 @@ class Item(abc.Row, abc.Parseable, table=schema.Item):
         "itemid": ("client_id", parse_integer),
         "status": ("status", str.lower),
     }
-    _pattern = re.compile(r"Infobox[\s_]Item")
+    _template = "Infobox_Object"
 
     __slots__ = (
         "article_id",
@@ -134,7 +134,7 @@ class Item(abc.Row, abc.Parseable, table=schema.Item):
         "value_buy",
         "weight",
         "item_class",
-        "type",
+        "item_type",
         "type_secondary",
         "flavor_text",
         "light_color",
@@ -149,9 +149,17 @@ class Item(abc.Row, abc.Parseable, table=schema.Item):
         "awarded_in",
         "sounds",
         "status",
+        "store_offers",
     )
 
     def __init__(self, **kwargs):
+        self.store_offers = []
+        self.dropped_by = []
+        self.bought_by = []
+        self.sold_by = []
+        self.sounds = []
+        self.awarded_in = []
+        self.attributes = []
         super().__init__(**kwargs)
 
     @property
@@ -264,54 +272,106 @@ class Item(abc.Row, abc.Parseable, table=schema.Item):
 
     @classmethod
     def from_article(cls, article):
-        item = super().from_article(article)
+        item: cls = super().from_article(article)
         if item is None:
             return None
+        if item.name is None:
+            item.name = item.title.lower()
         item.attributes = []
         for name, attribute in ItemAttribute._map.items():
             if attribute in item._raw_attributes and item._raw_attributes[attribute]:
                 item.attributes.append(ItemAttribute(item_id=item.article_id, name=name,
                                                      value=item._raw_attributes[attribute]))
-        if "attrib" in item._raw_attributes:
-            attribs = item._raw_attributes["attrib"].split(",")
-            for attr in attribs:
-                attr = attr.strip()
-                m = re.search(r'([\s\w]+)\s([+\-\d]+)', attr)
-                if m:
-                    attribute = m.group(1).replace("fighting", "").replace("level", "").strip()
-                    value = m.group(2)
-                    item.attributes.append(ItemAttribute(item_id=item.article_id, name=attribute, value=value))
-                if "regeneration" in attr:
-                    item.attributes.append(ItemAttribute(item_id=item.article_id, name="regeneration",
-                                                         value="faster regeneration"))
-        if "resist" in item._raw_attributes:
-            resistances = item._raw_attributes["resist"].split(",")
-            for element in resistances:
-                element = element.strip()
-                m = re.search(r'([a-zA-Z0-9_ ]+) +(-?\+?\d+)%', element)
-                if m:
-                    attribute = m.group(1) + "%"
-                    try:
-                        value = int(m.group(2))
-                    except ValueError:
-                        value = 0
-                    item.attributes.append(ItemAttribute(item_id=item.article_id, name=attribute, value=value))
+        item._parse_attributes()
+        item._parse_resists()
         vocations = item._raw_attributes.get('vocrequired')
         if vocations and "none" not in vocations.lower():
             vocation = vocations.replace('and', '+').replace(',', '+').replace(' ', '')
             item.attributes.append(ItemAttribute(item_id=item.article_id, name="vocation", value=vocation))
-        if "sounds" in item._raw_attributes:
-            sounds = parse_sounds(item._raw_attributes["sounds"])
-            if sounds:
-                item.sounds = [ItemSound(item_id=item.article_id, content=sound) for sound in sounds]
+        item._parse_sounds()
+        item._parse_store_value()
         return item
 
+    def _parse_attributes(self):
+        if "attrib" not in self._raw_attributes:
+            return
+        attribs = self._raw_attributes["attrib"].split(",")
+        for attr in attribs:
+            attr = attr.strip()
+            m = re.search(r'([\s\w]+)\s([+\-\d]+)', attr)
+            if "perfect shot" in attr.lower():
+                numbers = re.findall(r"(\d+)", attr)
+                if len(numbers) == 2:
+                    self.attributes.extend([
+                        ItemAttribute(item_id=self.article_id, name="perfect_shot", value=f"+{numbers[0]}"),
+                        ItemAttribute(item_id=self.article_id, name="perfect_shot_range", value=numbers[1]),
+                    ])
+                    continue
+            if "damage reflection" in attr.lower():
+                value = parse_integer(attr)
+                self.attributes.append(ItemAttribute(item_id=self.article_id, name="damage_reflection", value=value))
+            if "damage reflection" in attr.lower():
+                value = parse_integer(attr)
+                self.attributes.append(ItemAttribute(item_id=self.article_id, name="damage_reflection", value=value))
+            if "magic shield capacity" in attr.lower():
+                numbers = re.findall(r"(\d+)", attr)
+                if len(numbers) == 2:
+                    self.attributes.extend([
+                        ItemAttribute(item_id=self.article_id, name="magic_shield_capacity", value=f"+{numbers[0]}"),
+                        ItemAttribute(item_id=self.article_id, name="magic_shield_capacity%", value=f"{numbers[1]}%"),
+                    ])
+                    continue
+            if m:
+                attribute = m.group(1).replace("fighting", "").replace("level", "").strip().replace(" ", "_").lower()
+                value = m.group(2)
+                self.attributes.append(ItemAttribute(item_id=self.article_id, name=attribute.lower(), value=value))
+            if "regeneration" in attr:
+                self.attributes.append(ItemAttribute(item_id=self.article_id, name="regeneration",
+                                                     value="faster regeneration"))
+
+    def _parse_resists(self):
+        if "resist" not in self._raw_attributes:
+            return
+        resistances = self._raw_attributes["resist"].split(",")
+        for element in resistances:
+            element = element.strip()
+            m = re.search(r'([a-zA-Z0-9_ ]+) +(-?\+?\d+)%', element)
+            if not m:
+                continue
+            attribute = m.group(1) + "%"
+            try:
+                value = int(m.group(2))
+            except ValueError:
+                value = 0
+            self.attributes.append(ItemAttribute(item_id=self.article_id, name=attribute, value=value))
+
+    def _parse_sounds(self):
+        if "sounds" not in self._raw_attributes:
+            return
+        sounds = parse_sounds(self._raw_attributes["sounds"])
+        self.sounds = [ItemSound(item_id=self.article_id, content=sound) for sound in sounds] if sounds else []
+
+    def _parse_store_value(self):
+        if "storevalue" not in self._raw_attributes:
+            return
+        templates = find_templates(self._raw_attributes["storevalue"], "Store Product", recursive=True)
+        self.store_offers = []
+        for template in templates:
+            price = int(strip_code(template.get(1, 0)))
+            currency = strip_code(template.get(2, "Tibia Coin"))
+            amount = int(strip_code(template.get("amount", 1)))
+            self.store_offers.append(
+                ItemStoreOffer(item_id=self.article_id, price=price, currency=currency, amount=amount),
+            )
+
     def insert(self, c):
         super().insert(c)
         for attribute in getattr(self, "attributes", []):
             attribute.insert(c)
         for attribute in getattr(self, "sounds", []):
             attribute.insert(c)
+        for attribute in getattr(self, "store_offers", []):
+            attribute.insert(c)
 
     @classmethod
     def get_by_field(cls, c, field, value, use_like=False):
@@ -324,9 +384,93 @@ class Item(abc.Row, abc.Parseable, table=schema.Item):
         item.bought_by = NpcBuyOffer.search(c, "item_id", item.article_id, sort_by="value", ascending=False)
         item.awarded_in = QuestReward.search(c, "item_id", item.article_id)
         item.sounds = ItemSound.search(c, "item_id", item.article_id)
+        item.store_offers = ItemStoreOffer.search(c, "item_id", item.article_id)
         return item
 
 
+class Book(abc.Row, abc.Parseable, table=schema.Book):
+    """Represents a book or written document in Tibia.
+
+    Parameters
+    ----------
+    article_id: :class:`int`
+        The ID of the article that contains this book.
+    title: :class:`str`
+        The title of the article containing this book.
+    name: :class:´str´
+        The name of the book.
+    book_type: :class:`str`
+        The type of item this book can be found in.
+    item_id: :class:`int`
+        The ID of the item this book is written in.
+    location: :class:`str`
+        Where the book can be found.
+    blurb: :class:`str`
+        A short introduction text of the book.
+    author: :class:`str`
+        The person that wrote the book, if known.
+    prev_book: :class:`str`
+        If the book is part of a series, the book that precedes this one.
+    next_book: :class:`str`
+        If the book is part of a series, the book that follows this one.
+    text: :class:`str`
+        The content of the book.
+    version: :class:`str`
+        The client version where this item was first implemented.
+    status: :class:`str`
+        The status of this item in the game.
+    timestamp: :class:`int`
+        The last time the containing article was edited.
+    """
+
+    __slots__ = (
+        "article_id",
+        "title",
+        "name",
+        "book_type",
+        "item_id",
+        "location",
+        "blurb",
+        "author",
+        "prev_book",
+        "next_book",
+        "text",
+        "version",
+        "status",
+        "timestamp",
+    )
+
+    _map = {
+        "title": ("name", str),
+        "booktype": ("book_type", clean_links),
+        "location": ("location", lambda x: clean_links(x, True)),
+        "blurb": ("blurb", lambda x: clean_links(x, True)),
+        "author": ("author", lambda x: clean_links(x, True)),
+        "prevbook": ("prev_book", str),
+        "nextbook": ("next_book", str),
+        "text": ("text", clean_links),
+        "implemented": ("version", str),
+        "status": ("status", str),
+    }
+    _template = "Infobox_Book"
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def insert(self, c):
+        if getattr(self, "item_id", None):
+            super().insert(c)
+        else:
+            query = f"""
+                INSERT INTO {self.table.__tablename__}(article_id, title, name, book_type, item_id, location, blurb,
+                author, prev_book, next_book, text, version, status, timestamp)
+                VALUES(?, ?, ?, ?, (SELECT article_id FROM item WHERE title = ?), ?, ?, ?, ?, ?, ?, ?, ?, ?)
+            """
+            c.execute(query, (self.article_id, self.title, self.name, self.book_type, self.book_type, self.location,
+                              self.blurb, self.author, self.prev_book, self.next_book, self.text, self.version,
+                              self.status, self.timestamp))
+
+
 class Key(abc.Row, abc.Parseable, table=schema.ItemKey):
     """Represents a key item.
 
@@ -357,6 +501,7 @@ class Key(abc.Row, abc.Parseable, table=schema.ItemKey):
     version: :class:`str`
         The client version where this creature was first implemented.
     """
+
     __slots__ = (
         "article_id",
         "title",
@@ -381,7 +526,7 @@ class Key(abc.Row, abc.Parseable, table=schema.ItemKey):
         "implemented": ("version", str.strip),
         "status": ("status", str.lower),
     }
-    _pattern = re.compile(r"Infobox[\s_]Key")
+    _template = "Infobox_Key"
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
@@ -400,7 +545,7 @@ class Key(abc.Row, abc.Parseable, table=schema.ItemKey):
 
 
 class ItemAttribute(abc.Row, table=schema.ItemAttribute):
-    """Represents an Item's attribute
+    """Represents an Item's attribute.
 
     Attributes
     ----------
@@ -425,7 +570,7 @@ class ItemAttribute(abc.Row, table=schema.ItemAttribute):
         "hit%+": "hit_mod",
         "range": "range",
         "damage_type": "damagetype",
-        "damage": "damage",
+        "damage_range": "damagerange",
         "mana": "mana",
         "magic_level": "mlrequired",
         "words": "words",
@@ -443,6 +588,7 @@ class ItemAttribute(abc.Row, table=schema.ItemAttribute):
         "energy_attack": "energy_attack",
         "ice_attack": "ice_attack",
         "earth_attack": "earth_attack",
+        "weapon_type": "weapontype",
         "destructible": "destructible",
         "holds_liquid": "holdsliquid",
         "hangable": "hangable",
@@ -450,7 +596,12 @@ class ItemAttribute(abc.Row, table=schema.ItemAttribute):
         "rewritable": "rewritable",
         "writable_chars": "writechars",
         "consumable": "consumable",
-        "fansite": "fansite"
+        "fansite": "fansite",
+        "unshootable": "unshootable",
+        "blocks_path": "blockspath",
+        "walkable": "walkable",
+        "walking_speed": "walkingspeed",
+        "map_color": "mapcolor",
     }
     __slots__ = (
         "item_id",
@@ -497,3 +648,50 @@ class ItemSound(abc.Row, table=schema.ItemSound):
     def insert(self, c):
         columns = {'item_id': self.item_id, 'content': self.content}
         self.table.insert(c, **columns)
+
+
+class ItemStoreOffer(abc.Row, table=schema.ItemStoreOffer):
+    """Represents an offer for an item on the Tibia Store.
+
+    Attributes
+    ----------
+    item_id: :class:`int`
+        The ID of the item this offer is for.
+    price: :class:`int`
+        The price of the item.
+    amount: :class:`int`
+        The amount of the item.
+    currency: :class:`str`
+        The currency used. In most of the times it is Tibia Coins.
+    """
+
+    __slots__ = (
+        "item_id",
+        "price",
+        "amount",
+        "currency",
+    )
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def __repr__(self):
+        attributes = []
+        for attr in self.__slots__:
+            try:
+                v = getattr(self, attr)
+                if v is None:
+                    continue
+                attributes.append(f"{attr}={v!r}")
+            except AttributeError:
+                pass
+        return f"{self.__class__.__name__}({','.join(attributes)})"
+
+    def insert(self, c):
+        columns = {
+            'item_id': self.item_id,
+            'price': self.price,
+            'amount': self.amount or 1,
+            'currency': self.currency or 'Tibia Coin',
+        }
+        self.table.insert(c, **columns)
