@@ -12,7 +12,6 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-import re
 
 from tibiawikisql import schema
 from tibiawikisql.models import abc
@@ -20,12 +19,23 @@ from tibiawikisql.utils import parse_boolean, parse_integer, clean_links, client
 
 
 def remove_mount(name):
+    """Remove "(Mount)" from the name, if found.
+
+    Parameters
+    ----------
+    name: :class:`str`
+        The name to check.
+
+    Returns
+    -------
+    :class:`str`
+        The name with "(Mount)" removed from it.
+    """
     return name.replace("(Mount)", "").strip()
 
 
 class Mount(abc.Row, abc.Parseable, table=schema.Mount):
-    """
-    Represents a Game World.
+    """Represents a Game World.
 
     Attributes
     ----------
@@ -58,6 +68,7 @@ class Mount(abc.Row, abc.Parseable, table=schema.Mount):
     image: :class:`bytes`
         The NPC's image in bytes.
     """
+
     _map = {
         "name": ("name", remove_mount),
         "speed": ("speed", int),
@@ -71,7 +82,7 @@ class Mount(abc.Row, abc.Parseable, table=schema.Mount):
         "status": ("status", str.lower),
 
     }
-    _pattern = re.compile(r"Infobox[\s_]Mount")
+    _template = "Infobox_Mount"
     __slots__ = (
         "article_id",
         "title",
