@@ -12,6 +12,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+"""Defines the SQL schemas to use."""
+
 from tibiawikisql.database import Blob, Boolean, Column, ForeignKey, Integer, Real, Table, Text
 
 
@@ -25,6 +27,7 @@ class Achievement(Table):
     spoiler = Column(Text)
     secret = Column(Boolean)
     premium = Column(Boolean)
+    achievement_id = Column(Integer)
     version = Column(Text, index=True)
     status = Column(Text, default="active")
     timestamp = Column(Integer)
@@ -55,12 +58,11 @@ class Creature(Table):
     armor = Column(Integer)
     speed = Column(Integer)
     creature_class = Column(Text, index=True)
-    type = Column(Text, index=True)
+    creature_type = Column(Text, index=True)
     type_secondary = Column(Text, index=True)
     bestiary_class = Column(Text, index=True)
     bestiary_level = Column(Text, index=True)
     bestiary_occurrence = Column(Text, index=True)
-    max_damage = Column(Integer)
     runs_at = Column(Integer)
     summon_cost = Column(Integer)
     convince_cost = Column(Integer)
@@ -80,7 +82,6 @@ class Creature(Table):
     modifier_drown = Column(Integer)
     modifier_hpdrain = Column(Integer)
     modifier_healing = Column(Integer)
-    abilities = Column(Text)
     walks_through = Column(Text)
     walks_around = Column(Text)
     location = Column(Text)
@@ -90,6 +91,29 @@ class Creature(Table):
     timestamp = Column(Integer)
 
 
+class CreatureAbility(Table, table_name="creature_ability"):
+    creature_id = Column(ForeignKey(Integer, table="creature", column="article_id"), index=True)
+    name = Column(Text)
+    effect = Column(Text)
+    element = Column(Text)
+
+
+class CreatureMaxDamage(Table, table_name="creature_max_damage"):
+    creature_id = Column(ForeignKey(Integer, table="creature", column="article_id"), index=True)
+    physical = Column(Integer)
+    earth = Column(Integer)
+    fire = Column(Integer)
+    ice = Column(Integer)
+    energy = Column(Integer)
+    death = Column(Integer)
+    holy = Column(Integer)
+    drown = Column(Integer)
+    lifedrain = Column(Integer)
+    manadrain = Column(Integer)
+    summons = Column(Integer)
+    total = Column(Integer)
+
+
 class CreatureSound(Table, table_name="creature_sound"):
     creature_id = Column(ForeignKey(Integer, table="creature", column="article_id"), index=True)
     content = Column(Text)
@@ -108,7 +132,7 @@ class Item(Table):
     value_buy = Column(Integer)
     weight = Column(Real)
     item_class = Column(Text, index=True)
-    type = Column(Text, index=True)
+    item_type = Column(Text, index=True)
     type_secondary = Column(Text, index=True)
     flavor_text = Column(Text)
     light_color = Column(Integer)
@@ -125,6 +149,13 @@ class ItemSound(Table, table_name="item_sound"):
     content = Column(Text)
 
 
+class ItemStoreOffer(Table, table_name="item_store_offer"):
+    item_id = Column(ForeignKey(Integer, table="item", column="article_id"), index=True)
+    price = Column(Integer)
+    amount = Column(Integer)
+    currency = Column(Text)
+
+
 class CreatureDrop(Table, table_name="creature_drop"):
     creature_id = Column(ForeignKey(Integer, table="creature", column="article_id"), index=True)
     item_id = Column(ForeignKey(Integer, table="item", column="article_id"), index=True)
@@ -139,6 +170,23 @@ class ItemAttribute(Table, table_name="item_attribute"):
     value = Column(Text)
 
 
+class Book(Table):
+    article_id = Column(Integer, primary_key=True)
+    title = Column(Text, unique=True)
+    name = Column(Text)
+    book_type = Column(Text)
+    item_id = Column(ForeignKey(Integer, "item", "article_id"), index=True)
+    location = Column(Text)
+    blurb = Column(Text)
+    author = Column(Text)
+    prev_book = Column(Text)
+    next_book = Column(Text)
+    text = Column(Text)
+    version = Column(Text, index=True)
+    status = Column(Text, default="active")
+    timestamp = Column(Integer)
+
+
 class DatabaseInfo(Table, table_name="database_info"):
     key = Column(Text, primary_key=True)
     value = Column(Text)
@@ -240,12 +288,9 @@ class Npc(Table):
     article_id = Column(Integer, primary_key=True)
     title = Column(Text, unique=True, no_case=True)
     name = Column(Text, no_case=True, index=True)
-    race = Column(Text)
     gender = Column(Text, index=True)
     city = Column(Text, index=True)
     location = Column(Text)
-    job = Column(Text)
-    job_additionals = Column(Text)
     version = Column(Text, index=True)
     x = Column(Integer)
     y = Column(Integer)
@@ -255,6 +300,16 @@ class Npc(Table):
     timestamp = Column(Integer)
 
 
+class NpcJob(Table, table_name="npc_job"):
+    npc_id = Column(ForeignKey(Integer, "npc", "article_id"), index=True)
+    name = Column(Text, nullable=False)
+
+
+class NpcRace(Table, table_name="npc_race"):
+    npc_id = Column(ForeignKey(Integer, "npc", "article_id"), index=True)
+    name = Column(Text, nullable=False)
+
+
 class NpcBuying(Table, table_name="npc_offer_buy"):
     npc_id = Column(ForeignKey(Integer, "npc", "article_id"), index=True)
     item_id = Column(ForeignKey(Integer, "item", "article_id"), nullable=False, index=True)
@@ -404,12 +459,12 @@ class Update(Table, table_name="game_update"):
 
 
 def create_tables(conn):
-    """
-    Creates all the tables in the database.
+    """Create all the tables in the database.
+
     Parameters
     ----------
     conn: sqlite3.Connection, sqlite3.Cursor
-
+        A connection to the database.
     """
     for table in Table.all_tables():
         conn.execute(table.drop())
