@@ -13,26 +13,23 @@
 #  limitations under the License.
 import datetime
 import re
+from collections import defaultdict
+from typing import TYPE_CHECKING, List
+
+import mwparserfromhell
+from mwparserfromhell.nodes.extras import Parameter
+from mwparserfromhell.wikicode import Wikicode
+
+if TYPE_CHECKING:
+    from mwparserfromhell.nodes import Template
 
 min_max_pattern = re.compile(r"(\d+)-(\d+)")
-loot_stats_pattern = re.compile(r"\|([\s\w]+),\s*times:(\d+)(?:,\s*amount:([\d-]+))?")
-kills_pattern = re.compile(r"kills=(\d+)")
 int_pattern = re.compile(r"[+-]?\d+")
 float_pattern = re.compile(r'[+-]?(\d*[.])?\d+')
-named_links_pattern = re.compile(r'\[\[[^]|]+\|([^]]+)\]\]')
-file_pattern = re.compile(r'\[\[(?:File|Image):[^\]]+\]\]')
-
-links_pattern = re.compile(r'\[\[([^]]+)\]\]')
-external_links_pattern = re.compile(r'\[[^]]+\]')
-no_wiki_pattern = re.compile(r'<nowiki>([^<]+)</nowiki>')
-comments_pattern = re.compile(r'<!--.*-->', re.DOTALL | re.MULTILINE)
-
-sounds_template = re.compile(r"{{Sound List([^}]+)}}")
-sound_pattern = re.compile(r"\|([^|}]+)")
 
 
 def clean_question_mark(content):
-    """Removes question mark strings, turning them to nulls.
+    """Remove question mark strings, turning them to nulls.
 
     Parameters
     ----------
@@ -51,9 +48,8 @@ def clean_question_mark(content):
     return content.strip()
 
 
-def clean_links(content):
-    """
-    Removes any links from the string, changing them for their plan version.
+def clean_links(content, strip_question_mark=False):
+    """Remove any links from the string, changing them for their plan version.
 
     Parameters
     ----------
@@ -65,25 +61,16 @@ def clean_links(content):
     :class:`str`:
         The clean string, with no links.
     """
-    # Images
-    content = file_pattern.sub(' ', content)
-    # Named links
-    content = named_links_pattern.sub(r'\g<1>', content)
-    # Links
-    content = links_pattern.sub(r'\g<1>', content)
-    # External links
-    content = external_links_pattern.sub('', content)
-    # Double spaces
-    content = content.replace('  ', ' ')
-    # No wiki
-    content = no_wiki_pattern.sub(r'\g<1>', content)
-    # Comments
-    content = comments_pattern.sub('', content)
-    return content.strip()
+    content = re.sub(r"</?[bB][rR] ?/?>", "\n", content)
+    parsed = mwparserfromhell.parse(content)
+    content = parsed.strip_code().strip()
+    if strip_question_mark and content == "?":
+        return None
+    return content
 
 
 def convert_tibiawiki_position(pos):
-    """Converts from TibiaWiki position system to regular numeric coordinates
+    """Convert from TibiaWiki position system to regular numeric coordinates.
 
     TibiaWiki takes the coordinates and splits in two bytes, represented in decimal, separated by a period.
 
@@ -107,9 +94,65 @@ def convert_tibiawiki_position(pos):
         return 0
 
 
-def parse_boolean(value: str, default=False, invert=False):
+def find_template(content: str, template_name, partial=False, recursive=False):
+    """Find a template in a string containing wiki code.
+
+    If there are multiple matches, the first one will be returned.
+
+    Parameters
+    ----------
+    content: :class:`str`
+        A string containing wiki code.
+    template_name: :class:`str`
+        The name of the template to match. Case insensitive.
+    partial: :class:`bool`
+        Whether to match the entire template name or just a substring of it.
+
+        e.g. match "Loot Table" when searching for "Loot"
+    recursive: :class:`bool`
+        Whether to search for templates recursively, by going inside nested templates.
+
+    Returns
+    -------
+    :class:`Template`
+        The first template found in the content, if any. Otherwise ``None`` is returned.
     """
-    Parses a boolean value from a string.
+    return next(find_templates(content, template_name, partial, recursive), None)
+
+
+def find_templates(content: str, template_name, partial=False, recursive=False):
+    """Create a generator to find templates a wikicode string.
+
+    Parameters
+    ----------
+    content: :class:`str`
+        A string containing wiki code.
+    template_name: :class:`str`
+        The name of the template to match. Case insensitive.
+    partial: :class:`bool`
+        Whether to match the entire template name or just a substring of it.
+
+        e.g. match "Loot Table" when searching for "Loot"
+    recursive: :class:`bool`
+        Whether to search for templates recursively, by going inside nested templates.
+
+    Yields
+    ------
+    :class:`Template`
+        Templates matching provided string.
+    """
+    parsed = mwparserfromhell.parse(content)
+    templates: List['Template'] = parsed.ifilter_templates(recursive=recursive)
+    template_name = template_name.strip().lower().replace("_", " ")
+    for template in templates:
+        name = strip_code(template.name).lower().replace("_", " ")
+        if (partial and template_name in name) or (not partial and template_name == name):
+            yield template
+
+
+def parse_boolean(value: str, default=False, invert=False):
+    """Parse a boolean value from a string.
+
     String must contain "yes" to be considered True.
 
     Parameters
@@ -134,8 +177,9 @@ def parse_boolean(value: str, default=False, invert=False):
     else:
         return default
 
+
 def parse_date(value):
-    """Parse a date from the formats used in TibiaWiki
+    """Parse a date from the formats used in TibiaWiki.
 
     - June 28, 2019
     - Aug 21, 2014
@@ -182,8 +226,7 @@ def parse_float(value, default=0.0):
 
 
 def parse_integer(value, default=0):
-    """
-    Parses an integer from a string. Extra characters are ignored.
+    """Parse an integer from a string. Extra characters are ignored.
 
     Parameters
     ----------
@@ -205,8 +248,8 @@ def parse_integer(value, default=0):
 
 
 def parse_loot_statistics(value):
-    """
-    Gets every dropped item from a creature's loot statistics.
+    """Get every dropped item from a creature's loot statistics.
+
     Parameters
     ----------
     value: :class:`str`
@@ -217,16 +260,44 @@ def parse_loot_statistics(value):
     tuple:
         A tuple containing the total kills and a list of entries.
     """
-    match = kills_pattern.search(value)
-    if match:
-        return int(match.group(1)), loot_stats_pattern.findall(value)
-    else:
+    template = find_template(value, "Loot2", True)
+    if not template:
         return 0, []
+    kills = parse_integer(strip_code(template.get("kills", 0)))
+    entries = [_parse_loot_entry(param.value.strip_code()) for param in template.params if not param.showkey]
+    return kills, entries
+
+
+def _parse_loot_entry(entry):
+    """Parse a single parameter of the loot statistics template.
+
+    Parameters
+    ----------
+    entry: :class:`str`
+        A single item entry.
+
+    Returns
+    -------
+    :class:`dict`
+        A dictionary containing the drop data: item name, times dropped, amount dropped, etcetera.
+    """
+    arguments = entry.split(",")
+    entry = {"amount": "1"}
+    for arg in arguments:
+        subarg = arg.split(":")
+        if len(subarg) == 1:
+            key = "item"
+            value = arg.strip()
+        else:
+            key = subarg[0].strip()
+            value = subarg[1].strip()
+        entry[key] = value
+    return entry
 
 
 def parse_min_max(value):
-    """
-    Parses the mininum and maximum amounts of a loot drop.
+    """Parse the mininum and maximum amounts of a loot drop.
+
     They consist of two numbers separated by a hyphen, e.g. ``0-40``
 
     Parameters
@@ -247,7 +318,7 @@ def parse_min_max(value):
 
 
 def parse_sounds(value):
-    """Parses a list of sounds, using Template:Sound_List.
+    """Parse a list of sounds, using Template:Sound_List.
 
     Parameters
     ----------
@@ -257,15 +328,16 @@ def parse_sounds(value):
     Returns
     -------
     list:
-        A list of sounds."""
-    m = sounds_template.search(value)
-    if m:
-        return sound_pattern.findall(m.group(1))
-    return []
+        A list of sounds.
+    """
+    template = find_template(value, "Sound", True)
+    if not template:
+        return []
+    return [strip_code(param) for param in template.params]
 
 
 def client_color_to_rgb(value: int):
-    """Converts a color number from Tibia's client data to a RGB value.
+    """Convert a color number from Tibia's client data to a RGB value.
 
     Parameters
     ----------
@@ -275,15 +347,15 @@ def client_color_to_rgb(value: int):
     Returns
     -------
     int:
-        The hexadecimal color represented."""
+        The hexadecimal color represented.
+    """
     if value < 0 or value > 215:
         return 0
     return ((value // 36 * 0x33) << 16) + ((value // 6 % 6 * 0x33) << 8) + ((value % 6 * 0x33) & 0xFF)
 
 
-def parse_attributes(content):
-    """
-    Parses the attributes of an Infobox template.
+def parse_templatates_data(content):
+    """Parse the attributes of an Infobox template.
 
     Parameters
     ----------
@@ -295,40 +367,46 @@ def parse_attributes(content):
     :class:`dict[str, str]`:
         A dictionary with every attribute as key.
     """
-    attributes = {}
-    depth = 0
-    parse_value = False
-    attribute = ""
-    value = ""
-    for i in range(len(content)):
-        if content[i] == '{' or content[i] == '[':
-            depth += 1
-            if depth >= 3:
-                if parse_value:
-                    value = value + content[i]
-                else:
-                    attribute = attribute + content[i]
-        elif content[i] == '}' or content[i] == ']':
-            if depth >= 3:
-                if parse_value:
-                    value = value + content[i]
-                else:
-                    attribute = attribute + content[i]
-            if depth == 2:
-                attributes[attribute.strip()] = value.strip()
-                parse_value = False
-                attribute = ""
-                value = ""
-            depth -= 1
-        elif content[i] == '=' and depth == 2:
-            parse_value = True
-        elif content[i] == '|' and depth == 2:
-            attributes[attribute.strip()] = value.strip()
-            parse_value = False
-            attribute = ""
-            value = ""
-        elif parse_value:
-            value = value + content[i]
-        else:
-            attribute = attribute + content[i]
-    return {k: v.strip() for k, v in attributes.items() if v.strip()}
+    parsed = mwparserfromhell.parse(content)
+    templates = parsed.filter_templates(recursive=False)
+    if not templates:
+        return {}
+    data = defaultdict(dict)
+    for template in templates:
+        template_name = str(template.name).strip().replace(" ", "_")
+        for param in template.params:
+            key = param.name.strip()
+            if not param.showkey:
+                key = int(key)
+            value = param.value.strip()
+            if value:
+                data[template_name][key] = value
+    return data
+
+
+def strip_code(value):
+    """Strip code from Wikicode elements into plain strings.
+
+    Parameters
+    ----------
+    value:
+        A string or object containing wiki code.
+
+    Returns
+    -------
+    :class:`str`
+        A string representing the plain text content.
+    """
+    if value is None or isinstance(value, int):
+        return value
+    elif isinstance(value, str):
+        return value.strip()
+    elif isinstance(value, Parameter):
+        return value.value.strip_code().strip()
+    elif isinstance(value, Wikicode):
+        return value.strip_code().strip()
+    elif isinstance(value, dict):
+        for key, val in value.items():
+            value[key] = strip_code(val)
+        return value
+    return None
